package cd.casic.ci.process.process.service.stage.impl;

import cd.casic.ci.common.pipeline.constant.PipelineDateUtilConstant;
import cd.casic.ci.common.pipeline.container.Stage;
import cd.casic.ci.common.pipeline.req.stage.StageReq;
import cd.casic.ci.common.pipeline.resp.stage.StageResp;
import cd.casic.ci.common.pipeline.utils.PipelineDateUtil;
import cd.casic.ci.process.process.dal.pipeline.PipStageDao;
import cd.casic.ci.process.process.dataObject.stage.PipStage;
import cd.casic.ci.process.process.dataObject.task.PipTask;
import cd.casic.ci.process.process.service.stage.StageService;
import cd.casic.ci.process.process.service.task.TaskService;
import cd.casic.framework.commons.exception.ServiceException;
import cd.casic.framework.commons.exception.enums.GlobalErrorCodeConstants;
import cd.casic.framework.commons.util.object.BeanUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import jakarta.annotation.Resource;
import org.apache.commons.lang3.StringUtils;
import org.jsoup.helper.W3CDom;
import org.springframework.core.task.VirtualThreadTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

import static cd.casic.ci.process.constant.PipelineFinalConstant.TASK_TYPE_CODE;

@Service
public class StageServiceImpl implements StageService {
    @Resource
    private TaskService taskService;
    @Resource
    private PipStageDao stageDao;

    @Override
    public String createStagesOrTask(StageReq stageReq) {
        PipStage stage = new PipStage();
        BeanUtils.copyProperties(stageReq,stage);
        String stagesId = stage.getStageId();
        String pipelineId = stage.getPipelineId();

        String taskType = stage.getTaskType();
        int stageSort = stage.getStageSort();

        stage.setCreateTime(PipelineDateUtil.nowDateString(PipelineDateUtilConstant.yyyy_MM_dd_HH_mm_ss));

        PipTask tasks = new PipTask();
        tasks.setTaskType(taskType);
        tasks.setTaskSort(1);
        // TODO 查询是否为已知worker
        taskService.taskTypeExist(taskType);
        //是否为源码
        if (TASK_TYPE_CODE.equals(taskType)){
            //判断是否存在代码源(目标)节点
            findTargetTasks(pipelineId);

            //创建根节点
            int initStage = initStage(pipelineId,1);
            stage.setCode(true);
            stage.setStageSort(1);
            stage.setPipelineId(pipelineId);
            stage.setStageName("阶段-"+initStage);
            String id = createStages(stage);

            //从节点
            PipStage stages = new PipStage();
            stages.setStageSort(1);
            stages.setParentId(id);
            stages.setStageName("源码");
            stagesId = createStages(stages);
            tasks.setStageId(stagesId);

            return taskService.createTasksOrTask(tasks);
        }

        //新任务
        if (!StringUtils.isNotEmpty(stagesId) && stageSort != 0){
            //创建主节点
            int initStage = initStage(pipelineId, stageSort);
            stage.setStageSort(initStage);
            stage.setPipelineId(pipelineId);
            stage.setStageName("阶段-" + initStage);
            String id = createStages(stage);

            //创建从节点
            PipStage stages = new PipStage();
            stages.setParentId(id);
            stages.setStageSort(1);
            stages.setStageName("并行阶段-"+initStage+"-" + 1);
            stagesId = createStages(stages);
            tasks.setStageId(stagesId);
            return taskService.createTasksOrTask(tasks);
        }

        //并行任务
        if (StringUtils.isNotEmpty(stagesId)&& stageSort == 0){
            List<PipStage> otherStage = findSecondStage(stagesId);
            PipStage oneStages = findOneStages(stagesId);
            PipStage stages = new PipStage();
            stages.setParentId(stagesId);
            stages.setStageSort(otherStage.size() + 1);
            stages.setStageName("并行阶段-"+oneStages.getStageSort()+"-"+(otherStage.size()+1));
            stagesId = createStages(stages);
            tasks.setStageId(stagesId);
            return taskService.createTasksOrTask(tasks);
        }

        //串行任务
        int sort = stage.getTaskSort();
        if (sort != 0){
            tasks.setTaskSort(sort);
            tasks.setStageId(stagesId);
            return taskService.createTasksOrTask(tasks);
        }
        throw new ServiceException(50001,"未知的操作类型");
    }
    /**
     * 更新主节点阶段顺序
     * @param pipelineId 流水线id
     * @param taskSort 顺序
     * @return 顺序
     */
    private Integer initStage(String pipelineId,int taskSort){
        List<PipStage> allMainStage = findAllMainStage(pipelineId);
        if (allMainStage.isEmpty()){
            return 1;
        }
        for (PipStage stages : allMainStage) {
            int stage = stages.getStageSort();
            if (stage < taskSort){
                continue;
            }
            stages.setStageSort(stage+1);
            updateStages(stages);
        }
        return taskSort;
    }
    @Override
    public void createStageTemplate(String pipelineId, String[] template) {

    }

    @Override
    public void cloneStage(String pipelineId, String clonePipelineId) {

    }

    @Override
    public List<StageResp> findAllStagesOrTask(String pipelineId) {
        return null;
    }

    @Override
    public List<StageResp> findAllStagesTask(String pipelineId) {
        //获取流水线主节点
        List<PipStage> stageMainStage = findAllMainStage(pipelineId);
        if (stageMainStage.isEmpty()){
            return Collections.emptyList();
        }
        List<StageResp> list = new ArrayList<>();
        for (PipStage stage : stageMainStage) {
            String stagesId = stage.getStageId();
            //获取从节点
            List<PipStage> allStageStage = findOtherStageNoTask(stagesId);
            List<StageResp> stageRespList = allStageStage.stream().map(it -> {
                StageResp stageResp = new StageResp();
                BeanUtils.copyProperties(stageResp, it);
                return stageResp;
            }).toList();
            StageResp stageResp = new StageResp();
            BeanUtils.copyProperties(stage,stageResp);
            stageResp.setStageList(stageRespList);
            list.add(stageResp);
        }
        list.sort(Comparator.comparing(StageResp::getStageSort));
        return list;
    }

    @Override
    public void deleteStagesOrTask(String taskId) {
        PipTask taskQuery = new PipTask();
        taskQuery.setTaskId(taskId);
        List<PipTask> taskList = taskService.getTask(taskQuery);
        if (CollectionUtils.isEmpty(taskList)) {
            return;
        }
        PipTask task = taskList.get(0);
        taskService.deleteTaskById(taskId);
        // 查询上一级stage下有无其他task 没有则连着stage删除
        String stageId = task.getStageId();
        String pipelineId = task.getPipelineId();
        taskQuery.setTaskId("");
        taskQuery.setStageId(stageId);
        List<PipTask> otherTask = taskService.getTask(task);
        if (CollectionUtils.isEmpty(otherTask)) {
            // 删除当前task的父stage，然后判断父stage的父级有无其他子集如果没有就继续删除当前阶段

            PipStage stageQuery = new PipStage();
            stageQuery.setPipelineId(stageId);
            List<PipStage> currStageList = getPipStageList(stageQuery);
            if (CollectionUtils.isEmpty(currStageList)) {
                throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"stage不存在");
            }
            PipStage currStage = currStageList.get(0);
            deleteStages(stageId);
            String parentId = currStage.getParentId();
            stageQuery.setParentId(parentId);
            stageQuery.setStageId(null);
            // 查询同阶段其他二级stage，如果不存在其他stage则删除阶段stage并整理sort值
            List<PipStage> otherStageList = getPipStageList(stageQuery);
            if (CollectionUtils.isEmpty(otherStageList)) {
                //没有其他并行路径就需要删除当前阶段
                deleteStages(parentId);
            } else {
                for (PipStage stage : otherStageList) {
                    if (currStage.getStageSort()<stage.getStageSort()) {
                        stage.setStageSort(stage.getStageSort()-1);
                    }
                }
            }
        } else {
            for (PipTask pipTask : otherTask) {
                if (task.getTaskSort()<pipTask.getTaskSort()) {
                    pipTask.setTaskSort(pipTask.getTaskSort()-1);
                }
            }
        }

    }


    @Override
    public void deleteAllStagesOrTask(String pipelineId) {

    }

    private List<PipStage> getPipStageList(PipStage pipStage){
        LambdaQueryWrapper<PipStage> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(StringUtils.isNotEmpty(pipStage.getStageId()),PipStage::getStageId,pipStage.getStageId());
        wrapper.eq(StringUtils.isNotEmpty(pipStage.getPipelineId()),PipStage::getPipelineId,pipStage.getPipelineId());
        wrapper.eq(StringUtils.isNotEmpty(pipStage.getParentId()),PipStage::getParentId,pipStage.getParentId());
        return stageDao.selectList(wrapper);
    }

    @Override
    public void updateStageName(StageReq stage) {
        LambdaUpdateWrapper<PipStage> wrapper = new LambdaUpdateWrapper<>();
        if (StringUtils.isEmpty(stage.getStageName())||StringUtils.isEmpty(stage.getStageId())) {
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"阶段Id和名称蹦年为空");
        }
        wrapper.eq(PipStage::getStageId,stage.getStageId());
        wrapper.set(PipStage::getStageName,stage.getStageName());
        stageDao.update(wrapper);
    }

    @Override
    public List<PipStage> findAllMainStage(String pipelineId) {
        LambdaQueryWrapper<PipStage> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(PipStage::getPipelineId,pipelineId);
        wrapper.eq(PipStage::getParentId,"-1");
        return stageDao.selectList(wrapper);
    }

    @Override
    public void updateStagesTask(StageReq stage) {

    }

    @Override
    public List<String> validStagesMustField(String pipelineId) {
        return null;
    }

    @Override
    public String createStages(PipStage stage) {
        stageDao.insert(stage);
        return stage.getStageId();
    }

    @Override
    public void updateStages(PipStage stage) {
        stageDao.updateById(stage);
    }

    @Override
    public void deleteStages(String stageId) {
        stageDao.deleteById(stageId);
    }

    @Override
    public PipStage findOneStages(String stageId) {
        return null;
    }

    @Override
    public List<StageResp> findAllStages() {
        return null;
    }

    @Override
    public List<StageResp> findAllStagesList(List<String> idList) {
        return null;
    }
    /**
     * 判断任务是否存在代码源
     * @param pipelineId 流水线id
     */
    private void findTargetTasks(String pipelineId) {
        List<PipStage> allStage = findAllMainStage(pipelineId);
        if (allStage.isEmpty()){
            return;
        }
        for (PipStage stage : allStage) {
            if (stage.isCode()){
                throw new ServiceException(50001,"代码源已存在，无法再次创建。");
            }
        }
    }
    public List<PipStage> findOtherStageNoTask(String stagesId){
        List<PipStage> otherStage = findSecondStage(stagesId);
        List<PipStage> list = new ArrayList<>();
        for (PipStage stage : otherStage) {
            //获取阶段配置及任务
            String otherId = stage.getStageId();
            List<PipTask> allStageTask = taskService.finAllStageTask(otherId);
            stage.setTaskValues(allStageTask);
            list.add(stage);
        }
        return list;
    }
    // 获取主stage（阶段）下的子stage
    @Override
    public List<PipStage> findSecondStage(String stageId){
        LambdaQueryWrapper<PipStage> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(PipStage::getParentId,stageId);
        return stageDao.selectList(wrapper);
    }
}
